{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# Implementation\n",
    "\n",
    "## Example\n",
    "\n",
    "### Import modules"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {
    "tags": [
     "remove-output"
    ]
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "* Warning * Environment variable OMP_NUM_THREADS not set.\n",
      "* Warning * Setting OMP_NUM_THREADS to 4.\n"
     ]
    }
   ],
   "source": [
    "import veloxchem as vlx\n",
    "import numpy as np\n",
    "import py3Dmol as p3d"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Set up molecule and basis set"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {
    "tags": [
     "remove-output"
    ]
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "* Info * Reading basis set from file: /home/thomas/Notebook/anaconda/envs/echem/lib/python3.10/site-packages/veloxchem/basis/DEF2-SVP\n",
      "                                                                                                                          \n",
      "                                              Molecular Basis (Atomic Basis)                                              \n",
      "                                             ================================                                             \n",
      "                                                                                                                          \n",
      "                               Basis: DEF2-SVP                                                                            \n",
      "                                                                                                                          \n",
      "                               Atom Contracted GTOs           Primitive GTOs                                              \n",
      "                                                                                                                          \n",
      "                                H   (2S,1P)                   (4S,1P)                                                     \n",
      "                                C   (3S,2P,1D)                (7S,4P,1D)                                                  \n",
      "                                                                                                                          \n",
      "                               Contracted Basis Functions : 96                                                            \n",
      "                               Primitive Basis Functions  : 152                                                           \n",
      "                                                                                                                          \n"
     ]
    }
   ],
   "source": [
    "# molecule and basis\n",
    "\n",
    "mol_xyz = \"\"\"12\n",
    "c2h4-dimer\n",
    "C         -1.37731        1.01769       -0.71611\n",
    "C         -0.04211        1.07142       -0.72602\n",
    "H         -1.96225        1.74636       -0.16458\n",
    "H         -1.90859        0.23094       -1.24174\n",
    "H          0.49049        1.84498       -0.18262\n",
    "H          0.54315        0.32947       -1.25941\n",
    "C         -1.17537       -1.48468        2.37427\n",
    "C          0.06813       -1.06658        2.62697\n",
    "H         -1.35657       -2.40378        1.82687\n",
    "H          0.92893       -1.63558        2.29127\n",
    "H         -2.03527       -0.90348        2.69157\n",
    "H          0.24803       -0.13578        3.15527\n",
    "\"\"\"\n",
    "molecule = vlx.Molecule.read_xyz_string(mol_xyz)\n",
    "basis = vlx.MolecularBasis.read(molecule, 'def2-svp')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "metadata": {
    "tags": [
     "remove-input"
    ]
   },
   "outputs": [
    {
     "data": {
      "application/3dmoljs_load.v0": "<div id=\"3dmolviewer_167947558027661\"  style=\"position: relative; width: 300px; height: 300px\">\n        <p id=\"3dmolwarning_167947558027661\" style=\"background-color:#ffcccc;color:black\">You appear to be running in JupyterLab (or JavaScript failed to load for some other reason).  You need to install the 3dmol extension: <br>\n        <tt>jupyter labextension install jupyterlab_3dmol</tt></p>\n        </div>\n<script>\n\nvar loadScriptAsync = function(uri){\n  return new Promise((resolve, reject) => {\n    //this is to ignore the existence of requirejs amd\n    var savedexports, savedmodule;\n    if (typeof exports !== 'undefined') savedexports = exports;\n    else exports = {}\n    if (typeof module !== 'undefined') savedmodule = module;\n    else module = {}\n\n    var tag = document.createElement('script');\n    tag.src = uri;\n    tag.async = true;\n    tag.onload = () => {\n        exports = savedexports;\n        module = savedmodule;\n        resolve();\n    };\n  var firstScriptTag = document.getElementsByTagName('script')[0];\n  firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);\n});\n};\n\nif(typeof $3Dmolpromise === 'undefined') {\n$3Dmolpromise = null;\n  $3Dmolpromise = loadScriptAsync('https://cdnjs.cloudflare.com/ajax/libs/3Dmol/2.0.1/3Dmol-min.js');\n}\n\nvar viewer_167947558027661 = null;\nvar warn = document.getElementById(\"3dmolwarning_167947558027661\");\nif(warn) {\n    warn.parentNode.removeChild(warn);\n}\n$3Dmolpromise.then(function() {\nvar viewergrid_167947558027661 = null;\nviewergrid_167947558027661 = $3Dmol.createViewerGrid(document.getElementById(\"3dmolviewer_167947558027661\"),{rows: 1, cols: 1, control_all: true},{backgroundColor:\"white\"});\nviewer_167947558027661 = viewergrid_167947558027661[0][0];\nviewergrid_167947558027661[0][0].zoomTo();\tviewergrid_167947558027661[0][0].addModel(\"12\\nc2h4-dimer\\nC         -1.37731        1.01769       -0.71611\\nC         -0.04211        1.07142       -0.72602\\nH         -1.96225        1.74636       -0.16458\\nH         -1.90859        0.23094       -1.24174\\nH          0.49049        1.84498       -0.18262\\nH          0.54315        0.32947       -1.25941\\nC         -1.17537       -1.48468        2.37427\\nC          0.06813       -1.06658        2.62697\\nH         -1.35657       -2.40378        1.82687\\nH          0.92893       -1.63558        2.29127\\nH         -2.03527       -0.90348        2.69157\\nH          0.24803       -0.13578        3.15527\\n\",\"xyz\");\n\tviewergrid_167947558027661[0][0].setViewStyle({\"style\": \"outline\", \"width\": 0.05});\n\tviewergrid_167947558027661[0][0].setStyle({\"stick\": {}, \"sphere\": {\"scale\": 0.25}});\n\tviewergrid_167947558027661[0][0].rotate(90,\"x\");\nviewergrid_167947558027661[0][0].render();\n});\n</script>",
      "text/html": [
       "<div id=\"3dmolviewer_167947558027661\"  style=\"position: relative; width: 300px; height: 300px\">\n",
       "        <p id=\"3dmolwarning_167947558027661\" style=\"background-color:#ffcccc;color:black\">You appear to be running in JupyterLab (or JavaScript failed to load for some other reason).  You need to install the 3dmol extension: <br>\n",
       "        <tt>jupyter labextension install jupyterlab_3dmol</tt></p>\n",
       "        </div>\n",
       "<script>\n",
       "\n",
       "var loadScriptAsync = function(uri){\n",
       "  return new Promise((resolve, reject) => {\n",
       "    //this is to ignore the existence of requirejs amd\n",
       "    var savedexports, savedmodule;\n",
       "    if (typeof exports !== 'undefined') savedexports = exports;\n",
       "    else exports = {}\n",
       "    if (typeof module !== 'undefined') savedmodule = module;\n",
       "    else module = {}\n",
       "\n",
       "    var tag = document.createElement('script');\n",
       "    tag.src = uri;\n",
       "    tag.async = true;\n",
       "    tag.onload = () => {\n",
       "        exports = savedexports;\n",
       "        module = savedmodule;\n",
       "        resolve();\n",
       "    };\n",
       "  var firstScriptTag = document.getElementsByTagName('script')[0];\n",
       "  firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);\n",
       "});\n",
       "};\n",
       "\n",
       "if(typeof $3Dmolpromise === 'undefined') {\n",
       "$3Dmolpromise = null;\n",
       "  $3Dmolpromise = loadScriptAsync('https://cdnjs.cloudflare.com/ajax/libs/3Dmol/2.0.1/3Dmol-min.js');\n",
       "}\n",
       "\n",
       "var viewer_167947558027661 = null;\n",
       "var warn = document.getElementById(\"3dmolwarning_167947558027661\");\n",
       "if(warn) {\n",
       "    warn.parentNode.removeChild(warn);\n",
       "}\n",
       "$3Dmolpromise.then(function() {\n",
       "var viewergrid_167947558027661 = null;\n",
       "viewergrid_167947558027661 = $3Dmol.createViewerGrid(document.getElementById(\"3dmolviewer_167947558027661\"),{rows: 1, cols: 1, control_all: true},{backgroundColor:\"white\"});\n",
       "viewer_167947558027661 = viewergrid_167947558027661[0][0];\n",
       "viewergrid_167947558027661[0][0].zoomTo();\tviewergrid_167947558027661[0][0].addModel(\"12\\nc2h4-dimer\\nC         -1.37731        1.01769       -0.71611\\nC         -0.04211        1.07142       -0.72602\\nH         -1.96225        1.74636       -0.16458\\nH         -1.90859        0.23094       -1.24174\\nH          0.49049        1.84498       -0.18262\\nH          0.54315        0.32947       -1.25941\\nC         -1.17537       -1.48468        2.37427\\nC          0.06813       -1.06658        2.62697\\nH         -1.35657       -2.40378        1.82687\\nH          0.92893       -1.63558        2.29127\\nH         -2.03527       -0.90348        2.69157\\nH          0.24803       -0.13578        3.15527\\n\",\"xyz\");\n",
       "\tviewergrid_167947558027661[0][0].setViewStyle({\"style\": \"outline\", \"width\": 0.05});\n",
       "\tviewergrid_167947558027661[0][0].setStyle({\"stick\": {}, \"sphere\": {\"scale\": 0.25}});\n",
       "\tviewergrid_167947558027661[0][0].rotate(90,\"x\");\n",
       "viewergrid_167947558027661[0][0].render();\n",
       "});\n",
       "</script>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "viewer = p3d.view(viewergrid=(1,1),width=300,height=300)\n",
    "viewer.addModel(mol_xyz, 'xyz', viewer=(0,0))\n",
    "viewer.setViewStyle({\"style\": \"outline\", \"width\": 0.05})\n",
    "viewer.setStyle({\"stick\":{},\"sphere\": {\"scale\":0.25}})\n",
    "viewer.rotate(90,'x')\n",
    "viewer.show()"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Set up the exciton model driver"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "metadata": {},
   "outputs": [],
   "source": [
    "# exciton model setup\n",
    "exmod_settings = {\n",
    "    'fragments': '2',\n",
    "    'atoms_per_fragment': '6',\n",
    "    'charges': '0',\n",
    "    'nstates': '5',\n",
    "    'ct_nocc': '1',\n",
    "    'ct_nvir': '1',\n",
    "}\n",
    "\n",
    "method_settings = {'dft': 'no'}\n",
    "exmod_drv = vlx.ExcitonModelDriver()\n",
    "exmod_drv.update_settings(exmod_settings, method_settings)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Initializes exciton model Hamiltonian and transition dipoles"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "metadata": {},
   "outputs": [],
   "source": [
    "monomer_natoms = list(exmod_drv.natoms)\n",
    "n_monomers = len(monomer_natoms)\n",
    "monomer_start_indices = [sum(exmod_drv.natoms[:i]) for i in range(n_monomers)]\n",
    "\n",
    "npairs = n_monomers * (n_monomers - 1) // 2\n",
    "total_LE_states = n_monomers * exmod_drv.nstates\n",
    "total_CT_states = npairs * exmod_drv.ct_nocc * exmod_drv.ct_nvir * 2\n",
    "total_num_states = total_LE_states + total_CT_states\n",
    "\n",
    "exmod_drv.H = np.zeros((total_num_states, total_num_states))\n",
    "exmod_drv.elec_trans_dipoles = np.zeros((total_num_states, 3))\n",
    "exmod_drv.velo_trans_dipoles = np.zeros((total_num_states, 3))\n",
    "exmod_drv.magn_trans_dipoles = np.zeros((total_num_states, 3))\n",
    "exmod_drv.center_of_mass = molecule.center_of_mass()\n",
    "\n",
    "state_strings = ['' for s in range(total_num_states)]\n",
    "\n",
    "dimer_pairs = [(indA, indB)\n",
    "               for indA in range(n_monomers)\n",
    "               for indB in range(indA + 1, n_monomers)]\n",
    "excitation_ids = exmod_drv.get_excitation_ids(dimer_pairs)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Run monomer calculations"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "metadata": {
    "scrolled": true,
    "tags": [
     "remove-output"
    ]
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "                                              Molecular Geometry (Angstroms)                                              \n",
      "                                             ================================                                             \n",
      "                                                                                                                          \n",
      "                          Atom         Coordinate X          Coordinate Y          Coordinate Z                           \n",
      "                                                                                                                          \n",
      "                           C          -1.377310000000        1.017690000000       -0.716110000000                         \n",
      "                           C          -0.042110000000        1.071420000000       -0.726020000000                         \n",
      "                           H          -1.962250000000        1.746360000000       -0.164580000000                         \n",
      "                           H          -1.908590000000        0.230940000000       -1.241740000000                         \n",
      "                           H           0.490490000000        1.844980000000       -0.182620000000                         \n",
      "                           H           0.543150000000        0.329470000000       -1.259410000000                         \n",
      "                                                                                                                          \n",
      "                          Molecular charge            : 0                                                                 \n",
      "                          Spin multiplicity           : 1                                                                 \n",
      "                          Number of atoms             : 6                                                                 \n",
      "                          Number of alpha electrons   : 8                                                                 \n",
      "                          Number of beta  electrons   : 8                                                                 \n",
      "                                                                                                                          \n",
      "                                                                                                                          \n",
      "                                              Molecular Basis (Atomic Basis)                                              \n",
      "                                             ================================                                             \n",
      "                                                                                                                          \n",
      "                               Basis: DEF2-SVP                                                                            \n",
      "                                                                                                                          \n",
      "                               Atom Contracted GTOs           Primitive GTOs                                              \n",
      "                                                                                                                          \n",
      "                                H   (2S,1P)                   (4S,1P)                                                     \n",
      "                                C   (3S,2P,1D)                (7S,4P,1D)                                                  \n",
      "                                                                                                                          \n",
      "                               Contracted Basis Functions : 48                                                            \n",
      "                               Primitive Basis Functions  : 76                                                            \n",
      "                                                                                                                          \n",
      "                                                                                                                          \n",
      "                                            Self Consistent Field Driver Setup                                            \n",
      "                                           ====================================                                           \n",
      "                                                                                                                          \n",
      "                   Wave Function Model             : Spin-Restricted Hartree-Fock                                         \n",
      "                   Initial Guess Model             : Restart from Checkpoint                                              \n",
      "                   Convergence Accelerator         : Direct Inversion of Iterative Subspace                               \n",
      "                   Max. Number of Iterations       : 150                                                                  \n",
      "                   Max. Number of Error Vectors    : 10                                                                   \n",
      "                   Convergence Threshold           : 1.0e-06                                                              \n",
      "                   ERI Screening Scheme            : Cauchy Schwarz + Density                                             \n",
      "                   ERI Screening Mode              : Dynamic                                                              \n",
      "                   ERI Screening Threshold         : 1.0e-12                                                              \n",
      "                   Linear Dependence Threshold     : 1.0e-06                                                              \n",
      "                                                                                                                          \n",
      "* Info * Nuclear repulsion energy: 33.3198299171 a.u.                                                                     \n",
      "                                                                                                                          \n",
      "* Info * Overlap matrix computed in 0.00 sec.                                                                             \n",
      "                                                                                                                          \n",
      "* Info * Kinetic energy matrix computed in 0.00 sec.                                                                      \n",
      "                                                                                                                          \n",
      "* Info * Nuclear potential matrix computed in 0.00 sec.                                                                   \n",
      "                                                                                                                          \n",
      "* Info * Orthogonalization matrix computed in 0.02 sec.                                                                   \n",
      "                                                                                                                          \n",
      "* Info * Restarting from checkpoint file: monomer_1.scf.h5                                                                \n",
      "                                                                                                                          \n",
      "                                                                                                                          \n",
      "               Iter. | Hartree-Fock Energy | Energy Change | Gradient Norm | Max. Gradient | Density Change               \n",
      "               --------------------------------------------------------------------------------------------               \n",
      "                  1       -77.977546605109    0.0000000000      0.00000091      0.00000011      0.00000000                \n",
      "                                                                                                                          \n",
      "* Info * Checkpoint written to file: monomer_1.scf.h5                                                                     \n",
      "                                                                                                                          \n",
      "* Info * SCF tensors written to file: monomer_1.scf.tensors.h5                                                            \n",
      "                                                                                                                          \n",
      "               *** SCF converged in 1 iterations. Time: 0.40 sec.                                                         \n",
      "                                                                                                                          \n",
      "               Spin-Restricted Hartree-Fock:                                                                              \n",
      "               -----------------------------                                                                              \n",
      "               Total Energy                       :      -77.9775466051 a.u.                                              \n",
      "               Electronic Energy                  :     -111.2973765222 a.u.                                              \n",
      "               Nuclear Repulsion Energy           :       33.3198299171 a.u.                                              \n",
      "               ------------------------------------                                                                       \n",
      "               Gradient Norm                      :        0.0000009100 a.u.                                              \n",
      "                                                                                                                          \n",
      "                                                                                                                          \n",
      "               Ground State Information                                                                                   \n",
      "               ------------------------                                                                                   \n",
      "               Charge of Molecule            :  0.0                                                                       \n",
      "               Multiplicity (2S+1)           :  1.0                                                                       \n",
      "               Magnetic Quantum Number (M_S) :  0.0                                                                       \n",
      "                                                                                                                          \n",
      "                                                                                                                          \n",
      "                                                 Spin Restricted Orbitals                                                 \n",
      "                                                 ------------------------                                                 \n",
      "                                                                                                                          \n",
      "               Molecular Orbital No.   4:                                                                                 \n",
      "               --------------------------                                                                                 \n",
      "               Occupation: 2.000 Energy:   -0.78142 a.u.                                                                  \n",
      "               (   1 C   1s  :    -0.15) (   1 C   2s  :     0.30) (   1 C   1p+1:    -0.16)                              \n",
      "               (   2 C   1s  :     0.15) (   2 C   2s  :    -0.30) (   2 C   1p+1:    -0.16)                              \n",
      "               (   3 H   1s  :     0.20) (   4 H   1s  :     0.19) (   5 H   1s  :    -0.20)                              \n",
      "               (   6 H   1s  :    -0.19)                                                                                  \n",
      "                                                                                                                          \n",
      "               Molecular Orbital No.   5:                                                                                 \n",
      "               --------------------------                                                                                 \n",
      "               Occupation: 2.000 Energy:   -0.64291 a.u.                                                                  \n",
      "               (   1 C   1p-1:    -0.26) (   1 C   1p0 :    -0.18) (   2 C   1p-1:    -0.26)                              \n",
      "               (   2 C   1p0 :    -0.18) (   3 H   1s  :    -0.21) (   4 H   1s  :     0.21)                              \n",
      "               (   5 H   1s  :    -0.21) (   6 H   1s  :     0.21)                                                        \n",
      "                                                                                                                          \n",
      "               Molecular Orbital No.   6:                                                                                 \n",
      "               --------------------------                                                                                 \n",
      "               Occupation: 2.000 Energy:   -0.58489 a.u.                                                                  \n",
      "               (   1 C   1p+1:     0.41) (   2 C   1p+1:    -0.41) (   3 H   1s  :    -0.17)                              \n",
      "               (   4 H   1s  :    -0.17) (   5 H   1s  :    -0.17) (   6 H   1s  :    -0.17)                              \n",
      "                                                                                                                          \n",
      "               Molecular Orbital No.   7:                                                                                 \n",
      "               --------------------------                                                                                 \n",
      "               Occupation: 2.000 Energy:   -0.50726 a.u.                                                                  \n",
      "               (   1 C   1p-1:    -0.24) (   1 C   1p0 :    -0.17) (   2 C   1p-1:     0.24)                              \n",
      "               (   2 C   1p0 :     0.17) (   3 H   1s  :    -0.26) (   4 H   1s  :     0.26)                              \n",
      "               (   5 H   1s  :     0.26) (   6 H   1s  :    -0.26)                                                        \n",
      "                                                                                                                          \n",
      "               Molecular Orbital No.   8:                                                                                 \n",
      "               --------------------------                                                                                 \n",
      "               Occupation: 2.000 Energy:   -0.37604 a.u.                                                                  \n",
      "               (   1 C   1p-1:    -0.22) (   1 C   1p0 :     0.31) (   1 C   2p-1:    -0.17)                              \n",
      "               (   1 C   2p0 :     0.24) (   2 C   1p-1:    -0.22) (   2 C   1p0 :     0.31)                              \n",
      "               (   2 C   2p-1:    -0.17) (   2 C   2p0 :     0.24)                                                        \n",
      "                                                                                                                          \n",
      "               Molecular Orbital No.   9:                                                                                 \n",
      "               --------------------------                                                                                 \n",
      "               Occupation: 0.000 Energy:    0.16440 a.u.                                                                  \n",
      "               (   1 C   1p-1:    -0.20) (   1 C   1p0 :     0.28) (   1 C   2p-1:    -0.44)                              \n",
      "               (   1 C   2p0 :     0.62) (   2 C   1p-1:     0.20) (   2 C   1p0 :    -0.28)                              \n",
      "               (   2 C   2p-1:     0.44) (   2 C   2p0 :    -0.62)                                                        \n",
      "                                                                                                                          \n",
      "               Molecular Orbital No.  10:                                                                                 \n",
      "               --------------------------                                                                                 \n",
      "               Occupation: 0.000 Energy:    0.20503 a.u.                                                                  \n",
      "               (   1 C   3s  :     1.59) (   1 C   2p+1:    -0.56) (   2 C   3s  :     1.59)                              \n",
      "               (   2 C   2p+1:     0.56) (   3 H   2s  :    -1.12) (   4 H   2s  :    -1.12)                              \n",
      "               (   5 H   2s  :    -1.12) (   6 H   2s  :    -1.12)                                                        \n",
      "                                                                                                                          \n",
      "               Molecular Orbital No.  11:                                                                                 \n",
      "               --------------------------                                                                                 \n",
      "               Occupation: 0.000 Energy:    0.23177 a.u.                                                                  \n",
      "               (   1 C   1p-1:     0.18) (   1 C   2p-1:     0.63) (   1 C   2p0 :     0.44)                              \n",
      "               (   2 C   1p-1:     0.18) (   2 C   2p-1:     0.63) (   2 C   2p0 :     0.44)                              \n",
      "               (   3 H   2s  :    -1.17) (   4 H   2s  :     1.16) (   5 H   2s  :    -1.17)                              \n",
      "               (   6 H   2s  :     1.17)                                                                                  \n",
      "                                                                                                                          \n",
      "               Molecular Orbital No.  12:                                                                                 \n",
      "               --------------------------                                                                                 \n",
      "               Occupation: 0.000 Energy:    0.24119 a.u.                                                                  \n",
      "               (   1 C   3s  :     2.29) (   1 C   2p+1:    -0.19) (   2 C   3s  :    -2.29)                              \n",
      "               (   2 C   2p+1:    -0.19) (   3 H   2s  :    -1.27) (   4 H   2s  :    -1.27)                              \n",
      "               (   5 H   2s  :     1.27) (   6 H   2s  :     1.27)                                                        \n",
      "                                                                                                                          \n",
      "               Molecular Orbital No.  13:                                                                                 \n",
      "               --------------------------                                                                                 \n",
      "               Occupation: 0.000 Energy:    0.30329 a.u.                                                                  \n",
      "               (   1 C   1p-1:     0.16) (   1 C   2p-1:     1.31) (   1 C   2p0 :     0.93)                              \n",
      "               (   2 C   1p-1:    -0.16) (   2 C   2p-1:    -1.31) (   2 C   2p0 :    -0.93)                              \n",
      "               (   3 H   2s  :    -1.66) (   4 H   2s  :     1.66) (   5 H   2s  :     1.66)                              \n",
      "               (   6 H   2s  :    -1.66)                                                                                  \n",
      "                                                                                                                          \n",
      "                                                                                                                          \n",
      "                                                     TDA Driver Setup                                                     \n",
      "                                                    ==================                                                    \n",
      "                                                                                                                          \n",
      "                               Number of States                : 5                                                        \n",
      "                               Max. Number of Iterations       : 100                                                      \n",
      "                               Convergence Threshold           : 1.0e-04                                                  \n",
      "                               ERI Screening Scheme            : Cauchy Schwarz + Density                                 \n",
      "                               ERI Screening Threshold         : 1.0e-12                                                  \n",
      "                                                                                                                          \n",
      "* Info * Restarting from checkpoint file: monomer_1.rsp.h5                                                                \n",
      "                                                                                                                          \n",
      "                   *** Iteration:   1 * Reduced Space:    5 * Residues (Max,Min): 2.13e-01 and 6.34e-02                   \n",
      "                                                                                                                          \n",
      "                   State  1: 0.34776432 a.u. Residual Norm: 0.06411031                                                    \n",
      "                   State  2: 0.35563977 a.u. Residual Norm: 0.08504656                                                    \n",
      "                   State  3: 0.35615247 a.u. Residual Norm: 0.21331548                                                    \n",
      "                   State  4: 0.36532189 a.u. Residual Norm: 0.07855063                                                    \n",
      "                   State  5: 0.39640597 a.u. Residual Norm: 0.06336656                                                    \n",
      "                                                                                                                          \n",
      "                   *** Iteration:   2 * Reduced Space:   10 * Residues (Max,Min): 3.72e-02 and 9.22e-03                   \n",
      "                                                                                                                          \n",
      "                   State  1: 0.31097350 a.u. Residual Norm: 0.03718790                                                    \n",
      "                   State  2: 0.34145869 a.u. Residual Norm: 0.01285025                                                    \n",
      "                   State  3: 0.34478340 a.u. Residual Norm: 0.01055905                                                    \n",
      "                   State  4: 0.35566831 a.u. Residual Norm: 0.00921706                                                    \n",
      "                   State  5: 0.39016820 a.u. Residual Norm: 0.01267175                                                    \n",
      "                                                                                                                          \n",
      "                   *** Iteration:   3 * Reduced Space:   15 * Residues (Max,Min): 6.02e-02 and 1.94e-03                   \n",
      "                                                                                                                          \n",
      "                   State  1: 0.30903317 a.u. Residual Norm: 0.00798914                                                    \n",
      "                   State  2: 0.34134501 a.u. Residual Norm: 0.00248600                                                    \n",
      "                   State  3: 0.34465110 a.u. Residual Norm: 0.00201592                                                    \n",
      "                   State  4: 0.35557423 a.u. Residual Norm: 0.00194324                                                    \n",
      "                   State  5: 0.38822946 a.u. Residual Norm: 0.06022102                                                    \n",
      "                                                                                                                          \n",
      "                   *** Iteration:   4 * Reduced Space:   20 * Residues (Max,Min): 3.30e-02 and 1.24e-04                   \n",
      "                                                                                                                          \n",
      "                   State  1: 0.30897493 a.u. Residual Norm: 0.00322653                                                    \n",
      "                   State  2: 0.34133733 a.u. Residual Norm: 0.00047385                                                    \n",
      "                   State  3: 0.34464900 a.u. Residual Norm: 0.00012369                                                    \n",
      "                   State  4: 0.35557238 a.u. Residual Norm: 0.00013703                                                    \n",
      "                   State  5: 0.37684228 a.u. Residual Norm: 0.03299540                                                    \n",
      "                                                                                                                          \n",
      "                   *** Iteration:   5 * Reduced Space:   25 * Residues (Max,Min): 1.29e-02 and 5.35e-05                   \n",
      "                                                                                                                          \n",
      "                   State  1: 0.30897029 a.u. Residual Norm: 0.00084213                                                    \n",
      "                   State  2: 0.34133726 a.u. Residual Norm: 0.00011005                                                    \n",
      "                   State  3: 0.34464896 a.u. Residual Norm: 0.00007201                                                    \n",
      "                   State  4: 0.35557236 a.u. Residual Norm: 0.00005346                                                    \n",
      "                   State  5: 0.37565302 a.u. Residual Norm: 0.01293763                                                    \n",
      "                                                                                                                          \n",
      "                   *** Iteration:   6 * Reduced Space:   30 * Residues (Max,Min): 5.13e-03 and 7.94e-06                   \n",
      "                                                                                                                          \n",
      "                   State  1: 0.30897008 a.u. Residual Norm: 0.00010760                                                    \n",
      "                   State  2: 0.34133726 a.u. Residual Norm: 0.00002105                                                    \n",
      "                   State  3: 0.34464896 a.u. Residual Norm: 0.00001287                                                    \n",
      "                   State  4: 0.35557235 a.u. Residual Norm: 0.00000794                                                    \n",
      "                   State  5: 0.37556970 a.u. Residual Norm: 0.00512962                                                    \n",
      "                                                                                                                          \n",
      "                   *** Iteration:   7 * Reduced Space:   35 * Residues (Max,Min): 6.12e-04 and 8.41e-07                   \n",
      "                                                                                                                          \n",
      "                   State  1: 0.30897008 a.u. Residual Norm: 0.00001205                                                    \n",
      "                   State  2: 0.34133725 a.u. Residual Norm: 0.00000319                                                    \n",
      "                   State  3: 0.34464896 a.u. Residual Norm: 0.00000114                                                    \n",
      "                   State  4: 0.35557235 a.u. Residual Norm: 0.00000084                                                    \n",
      "                   State  5: 0.37556218 a.u. Residual Norm: 0.00061208                                                    \n",
      "                                                                                                                          \n",
      "                   *** Iteration:   8 * Reduced Space:   40 * Residues (Max,Min): 1.05e-04 and 3.30e-07                   \n",
      "                                                                                                                          \n",
      "                   State  1: 0.30897008 a.u. Residual Norm: 0.00000130                                                    \n",
      "                   State  2: 0.34133725 a.u. Residual Norm: 0.00000073                                                    \n",
      "                   State  3: 0.34464896 a.u. Residual Norm: 0.00000033                                                    \n",
      "                   State  4: 0.35557235 a.u. Residual Norm: 0.00000050                                                    \n",
      "                   State  5: 0.37556201 a.u. Residual Norm: 0.00010481                                                    \n",
      "                                                                                                                          \n",
      "                   *** Iteration:   9 * Reduced Space:   45 * Residues (Max,Min): 1.65e-05 and 1.20e-07                   \n",
      "                                                                                                                          \n",
      "                   State  1: 0.30897008 a.u. Residual Norm: 0.00000027                                                    \n",
      "                   State  2: 0.34133725 a.u. Residual Norm: 0.00000012                                                    \n",
      "                   State  3: 0.34464896 a.u. Residual Norm: 0.00000012                                                    \n",
      "                   State  4: 0.35557235 a.u. Residual Norm: 0.00000017                                                    \n",
      "                   State  5: 0.37556201 a.u. Residual Norm: 0.00001655                                                    \n",
      "                                                                                                                          \n",
      "               *** 5 excited states converged in 9 iterations. Time: 0.25 sec                                             \n",
      "                                                                                                                          \n",
      "                                                                                                                          \n",
      "* Info * Response solution vectors written to file: monomer_1.rsp.solutions.h5                                            \n",
      "                                                                                                                          \n",
      "               Electric Transition Dipole Moments (dipole length, a.u.)                                                   \n",
      "               --------------------------------------------------------                                                   \n",
      "                                                X            Y            Z                                               \n",
      "               Excited State    S1:      1.751428     0.070624    -0.013099                                               \n",
      "               Excited State    S2:     -0.004893     0.101652    -0.143186                                               \n",
      "               Excited State    S3:     -0.000404     0.005778     0.005701                                               \n",
      "               Excited State    S4:     -0.001038     0.004353     0.002742                                               \n",
      "               Excited State    S5:     -0.013278    -0.000576     0.000132                                               \n",
      "                                                                                                                          \n",
      "               Electric Transition Dipole Moments (dipole velocity, a.u.)                                                 \n",
      "               ----------------------------------------------------------                                                 \n",
      "                                                X            Y            Z                                               \n",
      "               Excited State    S1:      0.874293     0.035219    -0.006413                                               \n",
      "               Excited State    S2:     -0.008190     0.164623    -0.231994                                               \n",
      "               Excited State    S3:     -0.000168     0.003183     0.004934                                               \n",
      "               Excited State    S4:     -0.000697     0.003958     0.002264                                               \n",
      "               Excited State    S5:     -0.007697    -0.000362     0.000050                                               \n",
      "                                                                                                                          \n",
      "               Magnetic Transition Dipole Moments (a.u.)                                                                  \n",
      "               -----------------------------------------                                                                  \n",
      "                                                X            Y            Z                                               \n",
      "               Excited State    S1:      0.005420    -0.177719    -0.268587                                               \n",
      "               Excited State    S2:     -0.035857    -0.050997    -0.035103                                               \n",
      "               Excited State    S3:      0.588993     0.024477    -0.005151                                               \n",
      "               Excited State    S4:      0.405248     0.016471    -0.003944                                               \n",
      "               Excited State    S5:     -0.022746     0.663428     0.472983                                               \n",
      "                                                                                                                          \n",
      "               One-Photon Absorption                                                                                      \n",
      "               ---------------------                                                                                      \n",
      "               Excited State    S1:      0.30897008 a.u.      8.40750 eV    Osc.Str.    0.6329                            \n",
      "               Excited State    S2:      0.34133725 a.u.      9.28826 eV    Osc.Str.    0.0070                            \n",
      "               Excited State    S3:      0.34464896 a.u.      9.37838 eV    Osc.Str.    0.0000                            \n",
      "               Excited State    S4:      0.35557235 a.u.      9.67562 eV    Osc.Str.    0.0000                            \n",
      "               Excited State    S5:      0.37556201 a.u.     10.21956 eV    Osc.Str.    0.0000                            \n",
      "                                                                                                                          \n",
      "               Electronic Circular Dichroism                                                                              \n",
      "               -----------------------------                                                                              \n",
      "               Excited State    S1:     Rot.Str.      0.000202 a.u.     0.0953 [10**(-40) cgs]                            \n",
      "               Excited State    S2:     Rot.Str.      0.000042 a.u.     0.0198 [10**(-40) cgs]                            \n",
      "               Excited State    S3:     Rot.Str.     -0.000047 a.u.    -0.0220 [10**(-40) cgs]                            \n",
      "               Excited State    S4:     Rot.Str.     -0.000226 a.u.    -0.1066 [10**(-40) cgs]                            \n",
      "               Excited State    S5:     Rot.Str.     -0.000041 a.u.    -0.0194 [10**(-40) cgs]                            \n",
      "                                                                                                                          \n",
      "               Character of excitations:                                                                                  \n",
      "                                                                                                                          \n",
      "               Excited state 1                                                                                            \n",
      "               ---------------                                                                                            \n",
      "               HOMO     -> LUMO        -0.9680                                                                            \n",
      "                                                                                                                          \n",
      "               Excited state 2                                                                                            \n",
      "               ---------------                                                                                            \n",
      "               HOMO     -> LUMO+1       0.9939                                                                            \n",
      "                                                                                                                          \n",
      "               Excited state 3                                                                                            \n",
      "               ---------------                                                                                            \n",
      "               HOMO-1   -> LUMO        -0.7184                                                                            \n",
      "               HOMO     -> LUMO+2      -0.6807                                                                            \n",
      "                                                                                                                          \n",
      "               Excited state 4                                                                                            \n",
      "               ---------------                                                                                            \n",
      "               HOMO     -> LUMO+2       0.7161                                                                            \n",
      "               HOMO-1   -> LUMO        -0.6861                                                                            \n",
      "                                                                                                                          \n",
      "               Excited state 5                                                                                            \n",
      "               ---------------                                                                                            \n",
      "               HOMO-2   -> LUMO        -0.8776                                                                            \n",
      "               HOMO     -> LUMO+3      -0.4424                                                                            \n",
      "                                                                                                                          \n",
      "               Electric Transition Dipole Moments (dipole length, a.u.)                                                   \n",
      "               --------------------------------------------------------                                                   \n",
      "                                                X            Y            Z                                               \n",
      "               Excited State    S1:      1.751428     0.070624    -0.013099                                               \n",
      "               Excited State    S2:     -0.004893     0.101652    -0.143186                                               \n",
      "               Excited State    S3:     -0.000404     0.005778     0.005701                                               \n",
      "               Excited State    S4:     -0.001038     0.004353     0.002742                                               \n",
      "               Excited State    S5:     -0.013278    -0.000576     0.000132                                               \n",
      "                                                                                                                          \n",
      "               Electric Transition Dipole Moments (dipole velocity, a.u.)                                                 \n",
      "               ----------------------------------------------------------                                                 \n",
      "                                                X            Y            Z                                               \n",
      "               Excited State    S1:      0.874293     0.035219    -0.006413                                               \n",
      "               Excited State    S2:     -0.008190     0.164623    -0.231994                                               \n",
      "               Excited State    S3:     -0.000168     0.003183     0.004934                                               \n",
      "               Excited State    S4:     -0.000697     0.003958     0.002264                                               \n",
      "               Excited State    S5:     -0.007697    -0.000362     0.000050                                               \n",
      "                                                                                                                          \n",
      "               Magnetic Transition Dipole Moments (a.u.)                                                                  \n",
      "               -----------------------------------------                                                                  \n",
      "                                                X            Y            Z                                               \n",
      "               Excited State    S1:      0.005420    -0.177719    -0.268587                                               \n",
      "               Excited State    S2:     -0.035857    -0.050997    -0.035103                                               \n",
      "               Excited State    S3:      0.588993     0.024477    -0.005151                                               \n",
      "               Excited State    S4:      0.405248     0.016471    -0.003944                                               \n",
      "               Excited State    S5:     -0.022746     0.663428     0.472983                                               \n",
      "                                                                                                                          \n",
      "               One-Photon Absorption                                                                                      \n",
      "               ---------------------                                                                                      \n",
      "               Excited State    S1:      0.30897008 a.u.      8.40750 eV    Osc.Str.    0.6329                            \n",
      "               Excited State    S2:      0.34133725 a.u.      9.28826 eV    Osc.Str.    0.0070                            \n",
      "               Excited State    S3:      0.34464896 a.u.      9.37838 eV    Osc.Str.    0.0000                            \n",
      "               Excited State    S4:      0.35557235 a.u.      9.67562 eV    Osc.Str.    0.0000                            \n",
      "               Excited State    S5:      0.37556201 a.u.     10.21956 eV    Osc.Str.    0.0000                            \n",
      "                                                                                                                          \n",
      "               Electronic Circular Dichroism                                                                              \n",
      "               -----------------------------                                                                              \n",
      "               Excited State    S1:     Rot.Str.      0.000202 a.u.     0.0953 [10**(-40) cgs]                            \n",
      "               Excited State    S2:     Rot.Str.      0.000042 a.u.     0.0198 [10**(-40) cgs]                            \n",
      "               Excited State    S3:     Rot.Str.     -0.000047 a.u.    -0.0220 [10**(-40) cgs]                            \n",
      "               Excited State    S4:     Rot.Str.     -0.000226 a.u.    -0.1066 [10**(-40) cgs]                            \n",
      "               Excited State    S5:     Rot.Str.     -0.000041 a.u.    -0.0194 [10**(-40) cgs]                            \n",
      "                                                                                                                          \n",
      "               Character of excitations:                                                                                  \n",
      "                                                                                                                          \n",
      "               Excited state 1                                                                                            \n",
      "               ---------------                                                                                            \n",
      "               HOMO     -> LUMO        -0.9680                                                                            \n",
      "                                                                                                                          \n",
      "               Excited state 2                                                                                            \n",
      "               ---------------                                                                                            \n",
      "               HOMO     -> LUMO+1       0.9939                                                                            \n",
      "                                                                                                                          \n",
      "               Excited state 3                                                                                            \n",
      "               ---------------                                                                                            \n",
      "               HOMO-1   -> LUMO        -0.7184                                                                            \n",
      "               HOMO     -> LUMO+2      -0.6807                                                                            \n",
      "                                                                                                                          \n",
      "               Excited state 4                                                                                            \n",
      "               ---------------                                                                                            \n",
      "               HOMO     -> LUMO+2       0.7161                                                                            \n",
      "               HOMO-1   -> LUMO        -0.6861                                                                            \n",
      "                                                                                                                          \n",
      "               Excited state 5                                                                                            \n",
      "               ---------------                                                                                            \n",
      "               HOMO-2   -> LUMO        -0.8776                                                                            \n",
      "               HOMO     -> LUMO+3      -0.4424                                                                            \n",
      "                                                                                                                          \n",
      "                                              Molecular Geometry (Angstroms)                                              \n",
      "                                             ================================                                             \n",
      "                                                                                                                          \n",
      "                          Atom         Coordinate X          Coordinate Y          Coordinate Z                           \n",
      "                                                                                                                          \n",
      "                           C          -1.175370000000       -1.484680000000        2.374270000000                         \n",
      "                           C           0.068130000000       -1.066580000000        2.626970000000                         \n",
      "                           H          -1.356570000000       -2.403780000000        1.826870000000                         \n",
      "                           H           0.928930000000       -1.635580000000        2.291270000000                         \n",
      "                           H          -2.035270000000       -0.903480000000        2.691570000000                         \n",
      "                           H           0.248030000000       -0.135780000000        3.155270000000                         \n",
      "                                                                                                                          \n",
      "                          Molecular charge            : 0                                                                 \n",
      "                          Spin multiplicity           : 1                                                                 \n",
      "                          Number of atoms             : 6                                                                 \n",
      "                          Number of alpha electrons   : 8                                                                 \n",
      "                          Number of beta  electrons   : 8                                                                 \n",
      "                                                                                                                          \n",
      "                                                                                                                          \n",
      "                                              Molecular Basis (Atomic Basis)                                              \n",
      "                                             ================================                                             \n",
      "                                                                                                                          \n",
      "                               Basis: DEF2-SVP                                                                            \n",
      "                                                                                                                          \n",
      "                               Atom Contracted GTOs           Primitive GTOs                                              \n",
      "                                                                                                                          \n",
      "                                H   (2S,1P)                   (4S,1P)                                                     \n",
      "                                C   (3S,2P,1D)                (7S,4P,1D)                                                  \n",
      "                                                                                                                          \n",
      "                               Contracted Basis Functions : 48                                                            \n",
      "                               Primitive Basis Functions  : 76                                                            \n",
      "                                                                                                                          \n",
      "                                                                                                                          \n",
      "                                            Self Consistent Field Driver Setup                                            \n",
      "                                           ====================================                                           \n",
      "                                                                                                                          \n",
      "                   Wave Function Model             : Spin-Restricted Hartree-Fock                                         \n",
      "                   Initial Guess Model             : Restart from Checkpoint                                              \n",
      "                   Convergence Accelerator         : Direct Inversion of Iterative Subspace                               \n",
      "                   Max. Number of Iterations       : 150                                                                  \n",
      "                   Max. Number of Error Vectors    : 10                                                                   \n",
      "                   Convergence Threshold           : 1.0e-06                                                              \n",
      "                   ERI Screening Scheme            : Cauchy Schwarz + Density                                             \n",
      "                   ERI Screening Mode              : Dynamic                                                              \n",
      "                   ERI Screening Threshold         : 1.0e-12                                                              \n",
      "                   Linear Dependence Threshold     : 1.0e-06                                                              \n",
      "                                                                                                                          \n",
      "* Info * Nuclear repulsion energy: 33.3224196412 a.u.                                                                     \n",
      "                                                                                                                          \n",
      "* Info * Overlap matrix computed in 0.00 sec.                                                                             \n",
      "                                                                                                                          \n",
      "* Info * Kinetic energy matrix computed in 0.00 sec.                                                                      \n",
      "                                                                                                                          \n",
      "* Info * Nuclear potential matrix computed in 0.00 sec.                                                                   \n",
      "                                                                                                                          \n",
      "* Info * Orthogonalization matrix computed in 0.00 sec.                                                                   \n",
      "                                                                                                                          \n",
      "* Info * Restarting from checkpoint file: monomer_2.scf.h5                                                                \n",
      "                                                                                                                          \n",
      "                                                                                                                          \n",
      "               Iter. | Hartree-Fock Energy | Energy Change | Gradient Norm | Max. Gradient | Density Change               \n",
      "               --------------------------------------------------------------------------------------------               \n",
      "                  1       -77.977558810872    0.0000000000      0.00000094      0.00000011      0.00000000                \n",
      "                                                                                                                          \n",
      "* Info * Checkpoint written to file: monomer_2.scf.h5                                                                     \n",
      "                                                                                                                          \n",
      "* Info * SCF tensors written to file: monomer_2.scf.tensors.h5                                                            \n",
      "                                                                                                                          \n",
      "               *** SCF converged in 1 iterations. Time: 0.48 sec.                                                         \n",
      "                                                                                                                          \n",
      "               Spin-Restricted Hartree-Fock:                                                                              \n",
      "               -----------------------------                                                                              \n",
      "               Total Energy                       :      -77.9775588109 a.u.                                              \n",
      "               Electronic Energy                  :     -111.2999784521 a.u.                                              \n",
      "               Nuclear Repulsion Energy           :       33.3224196412 a.u.                                              \n",
      "               ------------------------------------                                                                       \n",
      "               Gradient Norm                      :        0.0000009393 a.u.                                              \n",
      "                                                                                                                          \n",
      "                                                                                                                          \n",
      "               Ground State Information                                                                                   \n",
      "               ------------------------                                                                                   \n",
      "               Charge of Molecule            :  0.0                                                                       \n",
      "               Multiplicity (2S+1)           :  1.0                                                                       \n",
      "               Magnetic Quantum Number (M_S) :  0.0                                                                       \n",
      "                                                                                                                          \n",
      "                                                                                                                          \n",
      "                                                 Spin Restricted Orbitals                                                 \n",
      "                                                 ------------------------                                                 \n",
      "                                                                                                                          \n",
      "               Molecular Orbital No.   4:                                                                                 \n",
      "               --------------------------                                                                                 \n",
      "               Occupation: 2.000 Energy:   -0.78137 a.u.                                                                  \n",
      "               (   1 C   1s  :     0.15) (   1 C   2s  :    -0.30) (   2 C   1s  :    -0.15)                              \n",
      "               (   2 C   2s  :     0.30) (   3 H   1s  :    -0.20) (   4 H   1s  :     0.20)                              \n",
      "               (   5 H   1s  :    -0.19) (   6 H   1s  :     0.19)                                                        \n",
      "                                                                                                                          \n",
      "               Molecular Orbital No.   5:                                                                                 \n",
      "               --------------------------                                                                                 \n",
      "               Occupation: 2.000 Energy:   -0.64290 a.u.                                                                  \n",
      "               (   1 C   1p-1:    -0.25) (   2 C   1p-1:    -0.25) (   3 H   1s  :     0.21)                              \n",
      "               (   4 H   1s  :     0.21) (   5 H   1s  :    -0.21) (   6 H   1s  :    -0.21)                              \n",
      "                                                                                                                          \n",
      "               Molecular Orbital No.   6:                                                                                 \n",
      "               --------------------------                                                                                 \n",
      "               Occupation: 2.000 Energy:   -0.58495 a.u.                                                                  \n",
      "               (   1 C   1p+1:     0.38) (   2 C   1p+1:    -0.38) (   3 H   1s  :    -0.17)                              \n",
      "               (   4 H   1s  :    -0.17) (   5 H   1s  :    -0.17) (   6 H   1s  :    -0.17)                              \n",
      "                                                                                                                          \n",
      "               Molecular Orbital No.   7:                                                                                 \n",
      "               --------------------------                                                                                 \n",
      "               Occupation: 2.000 Energy:   -0.50717 a.u.                                                                  \n",
      "               (   1 C   1p-1:     0.24) (   2 C   1p-1:    -0.24) (   3 H   1s  :    -0.26)                              \n",
      "               (   4 H   1s  :     0.26) (   5 H   1s  :     0.26) (   6 H   1s  :    -0.26)                              \n",
      "                                                                                                                          \n",
      "               Molecular Orbital No.   8:                                                                                 \n",
      "               --------------------------                                                                                 \n",
      "               Occupation: 2.000 Energy:   -0.37611 a.u.                                                                  \n",
      "               (   1 C   1p-1:    -0.19) (   1 C   1p0 :     0.33) (   1 C   2p0 :     0.26)                              \n",
      "               (   2 C   1p-1:    -0.19) (   2 C   1p0 :     0.33) (   2 C   2p0 :     0.26)                              \n",
      "                                                                                                                          \n",
      "               Molecular Orbital No.   9:                                                                                 \n",
      "               --------------------------                                                                                 \n",
      "               Occupation: 0.000 Energy:    0.16446 a.u.                                                                  \n",
      "               (   1 C   1p-1:    -0.17) (   1 C   1p0 :     0.30) (   1 C   2p-1:    -0.38)                              \n",
      "               (   1 C   2p0 :     0.65) (   2 C   1p-1:     0.17) (   2 C   1p0 :    -0.30)                              \n",
      "               (   2 C   2p-1:     0.38) (   2 C   2p0 :    -0.66)                                                        \n",
      "                                                                                                                          \n",
      "               Molecular Orbital No.  10:                                                                                 \n",
      "               --------------------------                                                                                 \n",
      "               Occupation: 0.000 Energy:    0.20503 a.u.                                                                  \n",
      "               (   1 C   3s  :    -1.59) (   1 C   2p+1:     0.52) (   1 C   2p-1:     0.17)                              \n",
      "               (   2 C   3s  :    -1.59) (   2 C   2p+1:    -0.52) (   2 C   2p-1:    -0.18)                              \n",
      "               (   3 H   2s  :     1.11) (   4 H   2s  :     1.11) (   5 H   2s  :     1.12)                              \n",
      "               (   6 H   2s  :     1.12)                                                                                  \n",
      "                                                                                                                          \n",
      "               Molecular Orbital No.  11:                                                                                 \n",
      "               --------------------------                                                                                 \n",
      "               Occupation: 0.000 Energy:    0.23175 a.u.                                                                  \n",
      "               (   1 C   1p-1:    -0.18) (   1 C   2p+1:     0.28) (   1 C   2p-1:    -0.62)                              \n",
      "               (   1 C   2p0 :    -0.36) (   2 C   1p-1:    -0.18) (   2 C   2p+1:     0.28)                              \n",
      "               (   2 C   2p-1:    -0.62) (   2 C   2p0 :    -0.36) (   3 H   2s  :    -1.17)                              \n",
      "               (   4 H   2s  :    -1.17) (   5 H   2s  :     1.16) (   6 H   2s  :     1.16)                              \n",
      "                                                                                                                          \n",
      "               Molecular Orbital No.  12:                                                                                 \n",
      "               --------------------------                                                                                 \n",
      "               Occupation: 0.000 Energy:    0.24118 a.u.                                                                  \n",
      "               (   1 C   3s  :    -2.29) (   1 C   2p+1:     0.18) (   2 C   3s  :     2.29)                              \n",
      "               (   2 C   2p+1:     0.18) (   3 H   2s  :     1.27) (   4 H   2s  :    -1.27)                              \n",
      "               (   5 H   2s  :     1.27) (   6 H   2s  :    -1.27)                                                        \n",
      "                                                                                                                          \n",
      "               Molecular Orbital No.  13:                                                                                 \n",
      "               --------------------------                                                                                 \n",
      "               Occupation: 0.000 Energy:    0.30331 a.u.                                                                  \n",
      "               (   1 C   1p-1:     0.16) (   1 C   2p+1:    -0.59) (   1 C   2p-1:     1.30)                              \n",
      "               (   1 C   2p0 :     0.75) (   2 C   1p-1:    -0.16) (   2 C   2p+1:     0.59)                              \n",
      "               (   2 C   2p-1:    -1.30) (   2 C   2p0 :    -0.75) (   3 H   2s  :     1.66)                              \n",
      "               (   4 H   2s  :    -1.66) (   5 H   2s  :    -1.66) (   6 H   2s  :     1.66)                              \n",
      "                                                                                                                          \n",
      "                                                                                                                          \n",
      "                                                     TDA Driver Setup                                                     \n",
      "                                                    ==================                                                    \n",
      "                                                                                                                          \n",
      "                               Number of States                : 5                                                        \n",
      "                               Max. Number of Iterations       : 100                                                      \n",
      "                               Convergence Threshold           : 1.0e-04                                                  \n",
      "                               ERI Screening Scheme            : Cauchy Schwarz + Density                                 \n",
      "                               ERI Screening Threshold         : 1.0e-12                                                  \n",
      "                                                                                                                          \n",
      "* Info * Restarting from checkpoint file: monomer_2.rsp.h5                                                                \n",
      "                                                                                                                          \n",
      "                   *** Iteration:   1 * Reduced Space:    5 * Residues (Max,Min): 2.13e-01 and 6.34e-02                   \n",
      "                                                                                                                          \n",
      "                   State  1: 0.34783116 a.u. Residual Norm: 0.06412244                                                    \n",
      "                   State  2: 0.35563877 a.u. Residual Norm: 0.08503436                                                    \n",
      "                   State  3: 0.35624616 a.u. Residual Norm: 0.21329400                                                    \n",
      "                   State  4: 0.36533374 a.u. Residual Norm: 0.07855921                                                    \n",
      "                   State  5: 0.39646091 a.u. Residual Norm: 0.06337301                                                    \n",
      "                                                                                                                          \n",
      "                   *** Iteration:   2 * Reduced Space:   10 * Residues (Max,Min): 3.72e-02 and 9.21e-03                   \n",
      "                                                                                                                          \n",
      "                   State  1: 0.31108179 a.u. Residual Norm: 0.03718485                                                    \n",
      "                   State  2: 0.34152241 a.u. Residual Norm: 0.01285172                                                    \n",
      "                   State  3: 0.34478504 a.u. Residual Norm: 0.01056213                                                    \n",
      "                   State  4: 0.35567255 a.u. Residual Norm: 0.00921130                                                    \n",
      "                   State  5: 0.39022130 a.u. Residual Norm: 0.01267120                                                    \n",
      "                                                                                                                          \n",
      "                   *** Iteration:   3 * Reduced Space:   15 * Residues (Max,Min): 6.01e-02 and 1.94e-03                   \n",
      "                                                                                                                          \n",
      "                   State  1: 0.30914159 a.u. Residual Norm: 0.00799112                                                    \n",
      "                   State  2: 0.34140869 a.u. Residual Norm: 0.00248645                                                    \n",
      "                   State  3: 0.34465274 a.u. Residual Norm: 0.00201444                                                    \n",
      "                   State  4: 0.35557853 a.u. Residual Norm: 0.00194458                                                    \n",
      "                   State  5: 0.38828574 a.u. Residual Norm: 0.06012338                                                    \n",
      "                                                                                                                          \n",
      "                   *** Iteration:   4 * Reduced Space:   20 * Residues (Max,Min): 3.29e-02 and 1.24e-04                   \n",
      "                                                                                                                          \n",
      "                   State  1: 0.30908321 a.u. Residual Norm: 0.00323357                                                    \n",
      "                   State  2: 0.34140101 a.u. Residual Norm: 0.00047406                                                    \n",
      "                   State  3: 0.34465065 a.u. Residual Norm: 0.00012395                                                    \n",
      "                   State  4: 0.35557668 a.u. Residual Norm: 0.00013733                                                    \n",
      "                   State  5: 0.37695764 a.u. Residual Norm: 0.03290224                                                    \n",
      "                                                                                                                          \n",
      "                   *** Iteration:   5 * Reduced Space:   25 * Residues (Max,Min): 1.29e-02 and 5.35e-05                   \n",
      "                                                                                                                          \n",
      "                   State  1: 0.30907855 a.u. Residual Norm: 0.00084350                                                    \n",
      "                   State  2: 0.34140093 a.u. Residual Norm: 0.00011023                                                    \n",
      "                   State  3: 0.34465061 a.u. Residual Norm: 0.00007207                                                    \n",
      "                   State  4: 0.35557665 a.u. Residual Norm: 0.00005353                                                    \n",
      "                   State  5: 0.37577701 a.u. Residual Norm: 0.01289342                                                    \n",
      "                                                                                                                          \n",
      "                   *** Iteration:   6 * Reduced Space:   30 * Residues (Max,Min): 5.11e-03 and 8.02e-06                   \n",
      "                                                                                                                          \n",
      "                   State  1: 0.30907834 a.u. Residual Norm: 0.00010769                                                    \n",
      "                   State  2: 0.34140093 a.u. Residual Norm: 0.00002109                                                    \n",
      "                   State  3: 0.34465060 a.u. Residual Norm: 0.00001291                                                    \n",
      "                   State  4: 0.35557665 a.u. Residual Norm: 0.00000802                                                    \n",
      "                   State  5: 0.37569441 a.u. Residual Norm: 0.00510829                                                    \n",
      "                                                                                                                          \n",
      "                   *** Iteration:   7 * Reduced Space:   35 * Residues (Max,Min): 6.12e-04 and 6.03e-07                   \n",
      "                                                                                                                          \n",
      "                   State  1: 0.30907834 a.u. Residual Norm: 0.00001208                                                    \n",
      "                   State  2: 0.34140093 a.u. Residual Norm: 0.00000320                                                    \n",
      "                   State  3: 0.34465060 a.u. Residual Norm: 0.00000104                                                    \n",
      "                   State  4: 0.35557665 a.u. Residual Norm: 0.00000060                                                    \n",
      "                   State  5: 0.37568693 a.u. Residual Norm: 0.00061164                                                    \n",
      "                                                                                                                          \n",
      "                   *** Iteration:   8 * Reduced Space:   40 * Residues (Max,Min): 1.06e-04 and 1.56e-07                   \n",
      "                                                                                                                          \n",
      "                   State  1: 0.30907834 a.u. Residual Norm: 0.00000134                                                    \n",
      "                   State  2: 0.34140093 a.u. Residual Norm: 0.00000074                                                    \n",
      "                   State  3: 0.34465060 a.u. Residual Norm: 0.00000016                                                    \n",
      "                   State  4: 0.35557665 a.u. Residual Norm: 0.00000022                                                    \n",
      "                   State  5: 0.37568677 a.u. Residual Norm: 0.00010635                                                    \n",
      "                                                                                                                          \n",
      "                   *** Iteration:   9 * Reduced Space:   45 * Residues (Max,Min): 2.38e-05 and 5.62e-08                   \n",
      "                                                                                                                          \n",
      "                   State  1: 0.30907834 a.u. Residual Norm: 0.00000038                                                    \n",
      "                   State  2: 0.34140093 a.u. Residual Norm: 0.00000020                                                    \n",
      "                   State  3: 0.34465060 a.u. Residual Norm: 0.00000006                                                    \n",
      "                   State  4: 0.35557665 a.u. Residual Norm: 0.00000010                                                    \n",
      "                   State  5: 0.37568676 a.u. Residual Norm: 0.00002378                                                    \n",
      "                                                                                                                          \n",
      "               *** 5 excited states converged in 9 iterations. Time: 0.26 sec                                             \n",
      "                                                                                                                          \n",
      "                                                                                                                          \n",
      "* Info * Response solution vectors written to file: monomer_2.rsp.solutions.h5                                            \n",
      "                                                                                                                          \n",
      "               Electric Transition Dipole Moments (dipole length, a.u.)                                                   \n",
      "               --------------------------------------------------------                                                   \n",
      "                                                X            Y            Z                                               \n",
      "               Excited State    S1:      1.631494     0.548438     0.331528                                               \n",
      "               Excited State    S2:      0.001132     0.088161    -0.151817                                               \n",
      "               Excited State    S3:     -0.002738     0.004884     0.006561                                               \n",
      "               Excited State    S4:     -0.001021     0.004831     0.001875                                               \n",
      "               Excited State    S5:     -0.012241    -0.004106    -0.002480                                               \n",
      "                                                                                                                          \n",
      "               Electric Transition Dipole Moments (dipole velocity, a.u.)                                                 \n",
      "               ----------------------------------------------------------                                                 \n",
      "                                                X            Y            Z                                               \n",
      "               Excited State    S1:      0.814914     0.273886     0.165526                                               \n",
      "               Excited State    S2:      0.001951     0.142769    -0.246050                                               \n",
      "               Excited State    S3:     -0.001953     0.001675     0.007254                                               \n",
      "               Excited State    S4:     -0.001115     0.004417     0.001220                                               \n",
      "               Excited State    S5:     -0.007104    -0.002345    -0.001426                                               \n",
      "                                                                                                                          \n",
      "               Magnetic Transition Dipole Moments (a.u.)                                                                  \n",
      "               -----------------------------------------                                                                  \n",
      "                                                X            Y            Z                                               \n",
      "               Excited State    S1:     -0.258067     0.614366     0.255234                                               \n",
      "               Excited State    S2:     -0.023319    -0.045429    -0.026723                                               \n",
      "               Excited State    S3:     -0.550911    -0.184525    -0.112430                                               \n",
      "               Excited State    S4:     -0.379258    -0.127485    -0.077885                                               \n",
      "               Excited State    S5:      0.299873    -0.660489    -0.379546                                               \n",
      "                                                                                                                          \n",
      "               One-Photon Absorption                                                                                      \n",
      "               ---------------------                                                                                      \n",
      "               Excited State    S1:      0.30907834 a.u.      8.41045 eV    Osc.Str.    0.6331                            \n",
      "               Excited State    S2:      0.34140093 a.u.      9.28999 eV    Osc.Str.    0.0070                            \n",
      "               Excited State    S3:      0.34465060 a.u.      9.37842 eV    Osc.Str.    0.0000                            \n",
      "               Excited State    S4:      0.35557665 a.u.      9.67573 eV    Osc.Str.    0.0000                            \n",
      "               Excited State    S5:      0.37568676 a.u.     10.22296 eV    Osc.Str.    0.0000                            \n",
      "                                                                                                                          \n",
      "               Electronic Circular Dichroism                                                                              \n",
      "               -----------------------------                                                                              \n",
      "               Excited State    S1:     Rot.Str.      0.000212 a.u.     0.0999 [10**(-40) cgs]                            \n",
      "               Excited State    S2:     Rot.Str.      0.000044 a.u.     0.0207 [10**(-40) cgs]                            \n",
      "               Excited State    S3:     Rot.Str.     -0.000049 a.u.    -0.0231 [10**(-40) cgs]                            \n",
      "               Excited State    S4:     Rot.Str.     -0.000235 a.u.    -0.1108 [10**(-40) cgs]                            \n",
      "               Excited State    S5:     Rot.Str.     -0.000040 a.u.    -0.0188 [10**(-40) cgs]                            \n",
      "                                                                                                                          \n",
      "               Character of excitations:                                                                                  \n",
      "                                                                                                                          \n",
      "               Excited state 1                                                                                            \n",
      "               ---------------                                                                                            \n",
      "               HOMO     -> LUMO        -0.9680                                                                            \n",
      "                                                                                                                          \n",
      "               Excited state 2                                                                                            \n",
      "               ---------------                                                                                            \n",
      "               HOMO     -> LUMO+1      -0.9937                                                                            \n",
      "                                                                                                                          \n",
      "               Excited state 3                                                                                            \n",
      "               ---------------                                                                                            \n",
      "               HOMO-1   -> LUMO        -0.7190                                                                            \n",
      "               HOMO     -> LUMO+2      -0.6800                                                                            \n",
      "                                                                                                                          \n",
      "               Excited state 4                                                                                            \n",
      "               ---------------                                                                                            \n",
      "               HOMO     -> LUMO+2       0.7167                                                                            \n",
      "               HOMO-1   -> LUMO        -0.6855                                                                            \n",
      "                                                                                                                          \n",
      "               Excited state 5                                                                                            \n",
      "               ---------------                                                                                            \n",
      "               HOMO-2   -> LUMO         0.8767                                                                            \n",
      "               HOMO     -> LUMO+3      -0.4442                                                                            \n",
      "                                                                                                                          \n",
      "               Electric Transition Dipole Moments (dipole length, a.u.)                                                   \n",
      "               --------------------------------------------------------                                                   \n",
      "                                                X            Y            Z                                               \n",
      "               Excited State    S1:      1.631494     0.548438     0.331528                                               \n",
      "               Excited State    S2:      0.001132     0.088161    -0.151817                                               \n",
      "               Excited State    S3:     -0.002738     0.004884     0.006561                                               \n",
      "               Excited State    S4:     -0.001021     0.004831     0.001875                                               \n",
      "               Excited State    S5:     -0.012241    -0.004106    -0.002480                                               \n",
      "                                                                                                                          \n",
      "               Electric Transition Dipole Moments (dipole velocity, a.u.)                                                 \n",
      "               ----------------------------------------------------------                                                 \n",
      "                                                X            Y            Z                                               \n",
      "               Excited State    S1:      0.814914     0.273886     0.165526                                               \n",
      "               Excited State    S2:      0.001951     0.142769    -0.246050                                               \n",
      "               Excited State    S3:     -0.001953     0.001675     0.007254                                               \n",
      "               Excited State    S4:     -0.001115     0.004417     0.001220                                               \n",
      "               Excited State    S5:     -0.007104    -0.002345    -0.001426                                               \n",
      "                                                                                                                          \n",
      "               Magnetic Transition Dipole Moments (a.u.)                                                                  \n",
      "               -----------------------------------------                                                                  \n",
      "                                                X            Y            Z                                               \n",
      "               Excited State    S1:     -0.258067     0.614366     0.255234                                               \n",
      "               Excited State    S2:     -0.023319    -0.045429    -0.026723                                               \n",
      "               Excited State    S3:     -0.550911    -0.184525    -0.112430                                               \n",
      "               Excited State    S4:     -0.379258    -0.127485    -0.077885                                               \n",
      "               Excited State    S5:      0.299873    -0.660489    -0.379546                                               \n",
      "                                                                                                                          \n",
      "               One-Photon Absorption                                                                                      \n",
      "               ---------------------                                                                                      \n",
      "               Excited State    S1:      0.30907834 a.u.      8.41045 eV    Osc.Str.    0.6331                            \n",
      "               Excited State    S2:      0.34140093 a.u.      9.28999 eV    Osc.Str.    0.0070                            \n",
      "               Excited State    S3:      0.34465060 a.u.      9.37842 eV    Osc.Str.    0.0000                            \n",
      "               Excited State    S4:      0.35557665 a.u.      9.67573 eV    Osc.Str.    0.0000                            \n",
      "               Excited State    S5:      0.37568676 a.u.     10.22296 eV    Osc.Str.    0.0000                            \n",
      "                                                                                                                          \n",
      "               Electronic Circular Dichroism                                                                              \n",
      "               -----------------------------                                                                              \n",
      "               Excited State    S1:     Rot.Str.      0.000212 a.u.     0.0999 [10**(-40) cgs]                            \n",
      "               Excited State    S2:     Rot.Str.      0.000044 a.u.     0.0207 [10**(-40) cgs]                            \n",
      "               Excited State    S3:     Rot.Str.     -0.000049 a.u.    -0.0231 [10**(-40) cgs]                            \n",
      "               Excited State    S4:     Rot.Str.     -0.000235 a.u.    -0.1108 [10**(-40) cgs]                            \n",
      "               Excited State    S5:     Rot.Str.     -0.000040 a.u.    -0.0188 [10**(-40) cgs]                            \n",
      "                                                                                                                          \n",
      "               Character of excitations:                                                                                  \n",
      "                                                                                                                          \n",
      "               Excited state 1                                                                                            \n",
      "               ---------------                                                                                            \n",
      "               HOMO     -> LUMO        -0.9680                                                                            \n",
      "                                                                                                                          \n",
      "               Excited state 2                                                                                            \n",
      "               ---------------                                                                                            \n",
      "               HOMO     -> LUMO+1      -0.9937                                                                            \n",
      "                                                                                                                          \n",
      "               Excited state 3                                                                                            \n",
      "               ---------------                                                                                            \n",
      "               HOMO-1   -> LUMO        -0.7190                                                                            \n",
      "               HOMO     -> LUMO+2      -0.6800                                                                            \n",
      "                                                                                                                          \n",
      "               Excited state 4                                                                                            \n",
      "               ---------------                                                                                            \n",
      "               HOMO     -> LUMO+2       0.7167                                                                            \n",
      "               HOMO-1   -> LUMO        -0.6855                                                                            \n",
      "                                                                                                                          \n",
      "               Excited state 5                                                                                            \n",
      "               ---------------                                                                                            \n",
      "               HOMO-2   -> LUMO         0.8767                                                                            \n",
      "               HOMO     -> LUMO+3      -0.4442                                                                            \n",
      "                                                                                                                          \n"
     ]
    }
   ],
   "source": [
    "# monomer calculations\n",
    "\n",
    "monomers_info = [{} for ind in range(n_monomers)]\n",
    "\n",
    "for ind in range(n_monomers):\n",
    "    monomer = molecule.get_sub_molecule(monomer_start_indices[ind],\n",
    "                                        monomer_natoms[ind])\n",
    "    monomer.set_charge(exmod_drv.charges[ind])\n",
    "    monomer.check_multiplicity()\n",
    "\n",
    "    scf_tensors = exmod_drv.monomer_scf(method_settings, ind, monomer, basis)\n",
    "    tda_results = exmod_drv.monomer_tda(method_settings, ind, monomer, basis,\n",
    "                                        scf_tensors)\n",
    "\n",
    "    monomers_info[ind]['mo'] = scf_tensors['C_alpha']\n",
    "    monomers_info[ind]['exc_energies'] = tda_results['exc_energies']\n",
    "    monomers_info[ind]['exc_vectors'] = tda_results['exc_vectors']\n",
    "\n",
    "    one_elec_ints = exmod_drv.get_one_elec_integrals(monomer, basis)\n",
    "    trans_dipoles = exmod_drv.get_LE_trans_dipoles(monomer, basis,\n",
    "                                                   one_elec_ints, scf_tensors,\n",
    "                                                   tda_results)\n",
    "\n",
    "    # LE states\n",
    "    for s in range(exmod_drv.nstates):\n",
    "        h = excitation_ids[ind, ind] + s\n",
    "        # LE energies\n",
    "        exmod_drv.H[h, h] = monomers_info[ind]['exc_energies'][s]\n",
    "        # LE transition dipoles\n",
    "        exmod_drv.elec_trans_dipoles[h, :] = trans_dipoles['electric'][s]\n",
    "        exmod_drv.velo_trans_dipoles[h, :] = trans_dipoles['velocity'][s]\n",
    "        exmod_drv.magn_trans_dipoles[h, :] = trans_dipoles['magnetic'][s]"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Run dimer calculations"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "metadata": {
    "scrolled": true,
    "tags": [
     "remove-output"
    ]
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "                                              Molecular Geometry (Angstroms)                                              \n",
      "                                             ================================                                             \n",
      "                                                                                                                          \n",
      "                          Atom         Coordinate X          Coordinate Y          Coordinate Z                           \n",
      "                                                                                                                          \n",
      "                           C          -1.377310000000        1.017690000000       -0.716110000000                         \n",
      "                           C          -0.042110000000        1.071420000000       -0.726020000000                         \n",
      "                           H          -1.962250000000        1.746360000000       -0.164580000000                         \n",
      "                           H          -1.908590000000        0.230940000000       -1.241740000000                         \n",
      "                           H           0.490490000000        1.844980000000       -0.182620000000                         \n",
      "                           H           0.543150000000        0.329470000000       -1.259410000000                         \n",
      "                           C          -1.175370000000       -1.484680000000        2.374270000000                         \n",
      "                           C           0.068130000000       -1.066580000000        2.626970000000                         \n",
      "                           H          -1.356570000000       -2.403780000000        1.826870000000                         \n",
      "                           H           0.928930000000       -1.635580000000        2.291270000000                         \n",
      "                           H          -2.035270000000       -0.903480000000        2.691570000000                         \n",
      "                           H           0.248030000000       -0.135780000000        3.155270000000                         \n",
      "                                                                                                                          \n",
      "                          Molecular charge            : 0                                                                 \n",
      "                          Spin multiplicity           : 1                                                                 \n",
      "                          Number of atoms             : 12                                                                \n",
      "                          Number of alpha electrons   : 16                                                                \n",
      "                          Number of beta  electrons   : 16                                                                \n",
      "                                                                                                                          \n",
      "                                                                                                                          \n",
      "                                              Molecular Basis (Atomic Basis)                                              \n",
      "                                             ================================                                             \n",
      "                                                                                                                          \n",
      "                               Basis: DEF2-SVP                                                                            \n",
      "                                                                                                                          \n",
      "                               Atom Contracted GTOs           Primitive GTOs                                              \n",
      "                                                                                                                          \n",
      "                                H   (2S,1P)                   (4S,1P)                                                     \n",
      "                                C   (3S,2P,1D)                (7S,4P,1D)                                                  \n",
      "                                                                                                                          \n",
      "                               Contracted Basis Functions : 96                                                            \n",
      "                               Primitive Basis Functions  : 152                                                           \n",
      "                                                                                                                          \n"
     ]
    }
   ],
   "source": [
    "# dimer calculations\n",
    "\n",
    "for ind_A in range(n_monomers):\n",
    "    monomer_A = molecule.get_sub_molecule(monomer_start_indices[ind_A],\n",
    "                                          monomer_natoms[ind_A])\n",
    "    monomer_A.set_charge(exmod_drv.charges[ind_A])\n",
    "    monomer_A.check_multiplicity()\n",
    "\n",
    "    for ind_B in range(ind_A + 1, n_monomers):\n",
    "        monomer_B = molecule.get_sub_molecule(monomer_start_indices[ind_B],\n",
    "                                              monomer_natoms[ind_B])\n",
    "        monomer_B.set_charge(exmod_drv.charges[ind_B])\n",
    "        monomer_B.check_multiplicity()\n",
    "\n",
    "        dimer = vlx.Molecule(monomer_A, monomer_B)\n",
    "        dimer.check_multiplicity()\n",
    "\n",
    "        mo_A = monomers_info[ind_A]['mo']\n",
    "        mo_B = monomers_info[ind_B]['mo']\n",
    "\n",
    "        nocc_A = monomer_A.number_of_alpha_electrons()\n",
    "        nocc_B = monomer_B.number_of_alpha_electrons()\n",
    "        nvir_A = mo_A.shape[1] - nocc_A\n",
    "        nvir_B = mo_B.shape[1] - nocc_B\n",
    "\n",
    "        nocc = nocc_A + nocc_B\n",
    "        nvir = nvir_A + nvir_B\n",
    "\n",
    "        mo = exmod_drv.dimer_mo_coefficients(monomer_A, monomer_B, basis, mo_A,\n",
    "                                             mo_B)\n",
    "\n",
    "        dimer_prop = exmod_drv.dimer_properties(dimer, basis, mo)\n",
    "\n",
    "        dimer_energy = dimer_prop['energy']\n",
    "\n",
    "        exc_vectors_A = monomers_info[ind_A]['exc_vectors']\n",
    "        exc_vectors_B = monomers_info[ind_B]['exc_vectors']\n",
    "\n",
    "        exc_vectors = []\n",
    "\n",
    "        exc_vectors += exmod_drv.dimer_excitation_vectors_LE_A(\n",
    "            exc_vectors_A, ind_A, nocc_A, nvir_A, nocc, nvir, excitation_ids)\n",
    "\n",
    "        exc_vectors += exmod_drv.dimer_excitation_vectors_LE_B(\n",
    "            exc_vectors_B, ind_B, nocc_A, nvir_A, nocc, nvir, excitation_ids)\n",
    "\n",
    "        exc_vectors += exmod_drv.dimer_excitation_vectors_CT_AB(\n",
    "            ind_A, ind_B, nocc_A, nvir_A, nocc, nvir, excitation_ids)\n",
    "\n",
    "        exc_vectors += exmod_drv.dimer_excitation_vectors_CT_BA(\n",
    "            ind_A, ind_B, nocc_A, nvir_A, nocc, nvir, excitation_ids)\n",
    "        \n",
    "        for c_vec in exc_vectors:\n",
    "            state_strings[c_vec['index']] = c_vec['type'] + '(' + c_vec['frag'] + ')'\n",
    "            state_strings[c_vec['index']] += '   ' + c_vec['name']\n",
    "\n",
    "        sigma_vectors = exmod_drv.dimer_sigma_vectors(dimer, basis, dimer_prop,\n",
    "                                                      mo, exc_vectors)\n",
    "\n",
    "        one_elec_ints = exmod_drv.get_one_elec_integrals(dimer, basis)\n",
    "        trans_dipoles = exmod_drv.get_CT_trans_dipoles(\n",
    "            dimer, basis, one_elec_ints, mo,\n",
    "            exc_vectors[exmod_drv.nstates * 2:])\n",
    "\n",
    "        # CT states\n",
    "        for i_vec, (c_vec, s_vec) in enumerate(\n",
    "                zip(exc_vectors[exmod_drv.nstates * 2:],\n",
    "                    sigma_vectors[exmod_drv.nstates * 2:])):\n",
    "            # CT energies\n",
    "            energy = np.vdot(c_vec['vec'], s_vec['vec'])\n",
    "            exmod_drv.H[c_vec['index'], c_vec['index']] = energy\n",
    "            # CT transition dipoles\n",
    "            exmod_drv.elec_trans_dipoles[\n",
    "                c_vec['index'], :] = trans_dipoles['electric'][i_vec]\n",
    "            exmod_drv.velo_trans_dipoles[\n",
    "                c_vec['index'], :] = trans_dipoles['velocity'][i_vec]\n",
    "            exmod_drv.magn_trans_dipoles[\n",
    "                c_vec['index'], :] = trans_dipoles['magnetic'][i_vec]\n",
    "\n",
    "        # LE(A)-LE(B) couplings\n",
    "        for c_vec in exc_vectors[:exmod_drv.nstates]:\n",
    "            for s_vec in sigma_vectors[exmod_drv.nstates:exmod_drv.nstates * 2]:\n",
    "                coupling = np.vdot(c_vec['vec'], s_vec['vec'])\n",
    "                exmod_drv.H[c_vec['index'], s_vec['index']] = coupling\n",
    "                exmod_drv.H[s_vec['index'], c_vec['index']] = coupling\n",
    "\n",
    "        # LE-CT couplings\n",
    "        for c_vec in exc_vectors[:exmod_drv.nstates * 2]:\n",
    "            for s_vec in sigma_vectors[exmod_drv.nstates * 2:]:\n",
    "                coupling = np.vdot(c_vec['vec'], s_vec['vec'])\n",
    "                exmod_drv.H[c_vec['index'], s_vec['index']] = coupling\n",
    "                exmod_drv.H[s_vec['index'], c_vec['index']] = coupling\n",
    "\n",
    "        # CT-CT couplings\n",
    "        for c_vec in exc_vectors[exmod_drv.nstates * 2:]:\n",
    "            for s_vec in sigma_vectors[exmod_drv.nstates * 2:]:\n",
    "                if c_vec['index'] >= s_vec['index']:\n",
    "                    continue\n",
    "                coupling = np.vdot(c_vec['vec'], s_vec['vec'])\n",
    "                exmod_drv.H[c_vec['index'], s_vec['index']] = coupling\n",
    "                exmod_drv.H[s_vec['index'], c_vec['index']] = coupling"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Print the exciton model Hamiltonian"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {
    "tags": [
     "output_scroll"
    ]
   },
   "outputs": [],
   "source": [
    "np.set_printoptions(precision=6, suppress=True)\n",
    "\n",
    "n = exmod_drv.nstates\n",
    "H = exmod_drv.H\n",
    "\n",
    "print('The LE(A)-LE(A) block:\\n')\n",
    "print(H[:n, :n], '\\n')\n",
    "\n",
    "print('The LE(A)-LE(B) block:\\n')\n",
    "print(H[:n, n:n*2], '\\n')\n",
    "\n",
    "print('The LE(A)-CT block:\\n')\n",
    "print(H[:n, n*2:], '\\n')\n",
    "\n",
    "print('The LE(B)-LE(B) block:\\n')\n",
    "print(H[n:n*2, n:n*2], '\\n')\n",
    "\n",
    "print('The LE(B)-CT block:\\n')\n",
    "print(H[n:n*2, n*2:], '\\n')\n",
    "\n",
    "print('The CT-CT block:\\n')\n",
    "print(H[n*2:, n*2:], '\\n')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "for s, state_str in enumerate(state_strings):\n",
    "    print(f'diabatic state {s+1:<5d}', state_str)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Get excitation energies and transition dipoles"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "# Exciton model energies\n",
    "\n",
    "eigvals, eigvecs = np.linalg.eigh(exmod_drv.H)\n",
    "\n",
    "elec_trans_dipoles = np.matmul(eigvecs.T, exmod_drv.elec_trans_dipoles)\n",
    "velo_trans_dipoles = np.matmul(eigvecs.T, exmod_drv.velo_trans_dipoles)\n",
    "magn_trans_dipoles = np.matmul(eigvecs.T, exmod_drv.magn_trans_dipoles)\n",
    "\n",
    "excitation_energies = []\n",
    "oscillator_strengths = []\n",
    "rotatory_strengths = []\n",
    "\n",
    "for s in range(total_num_states):\n",
    "    ene = eigvals[s]\n",
    "    dip_strength = np.sum(elec_trans_dipoles[s, :]**2)\n",
    "    f = (2.0 / 3.0) * dip_strength * ene\n",
    "\n",
    "    velo_trans_dipoles[s, :] /= ene\n",
    "    R = np.vdot(velo_trans_dipoles[s, :], magn_trans_dipoles[s, :])\n",
    "    \n",
    "    excitation_energies.append(ene)\n",
    "    oscillator_strengths.append(f)\n",
    "    rotatory_strengths.append(R)\n",
    "\n",
    "    print(f'S{s+1:<2d}  {ene*vlx.hartree_in_ev():10.5f} eV  f={f:<.4f}  R={R:<.4f}')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {
    "tags": [
     "output_scroll"
    ]
   },
   "outputs": [],
   "source": [
    "for s in range(total_num_states):\n",
    "    print(f'S{s+1}:')\n",
    "    c_squared = eigvecs[:, s]**2\n",
    "    components = []\n",
    "    for c2, state_str in zip(c_squared, state_strings):\n",
    "        if c2 > 0.04:\n",
    "            components.append((c2, state_str))\n",
    "    for c2, state_str in sorted(components, reverse=True):\n",
    "        print(f'    {c2*100:5.1f}%  {state_str}')"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Plot absorption and ECD spectra"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "import matplotlib.pyplot as plt\n",
    "\n",
    "def lorentzian(x, y, xmin, xmax, xstep, gamma):\n",
    "    '''\n",
    "    Lorentzian broadening function\n",
    "    \n",
    "    Call: xi,yi = lorentzian(energies, intensities, start energy, end energy, energy step, gamma)\n",
    "    '''\n",
    "    xi = np.arange(xmin,xmax,xstep); yi=np.zeros(len(xi))\n",
    "    for i in range(len(xi)):\n",
    "        for k in range(len(x)): yi[i] = yi[i] + y[k] * (gamma/2.) / ( (xi[i]-x[k])**2 + (gamma/2.)**2 )\n",
    "    return xi,yi\n",
    "\n",
    "plt.figure(figsize=(6,4))\n",
    "\n",
    "x = np.array(excitation_energies) * 27.211385\n",
    "y_abs = np.array(oscillator_strengths)\n",
    "y_ecd = np.array(rotatory_strengths)\n",
    "\n",
    "x0,y0 = lorentzian(x, y_abs, min(x)-1.0, max(x)+1.0, 0.01, 0.2)\n",
    "plt.plot(x0,y0)\n",
    "plt.show()\n",
    "\n",
    "x1,y1 = lorentzian(x, y_ecd, min(x)-1.0, max(x)+1.0, 0.01, 0.2)\n",
    "plt.plot(x1,y1)\n",
    "plt.show()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "x_ref = 27.211385 * np.array([\n",
    "    0.29347889, 0.31661749, 0.33498928, 0.33948716, 0.34142481, 0.34378921,\n",
    "    0.35317567, 0.35467501, 0.37317654, 0.37523192, 0.38737236, 0.38809457,\n",
    "    0.40824740, 0.41131899, 0.44405319, 0.44836638, 0.45442440, 0.45618961,\n",
    "    0.45869117, 0.46343516, 0.46961985, 0.47008569, 0.48617428, 0.48913889,\n",
    "    0.49490201, 0.49583015, 0.49623637, 0.50093006, 0.50196948, 0.50276614,\n",
    "    0.54606024, 0.54653988, 0.54825489, 0.54984647, 0.55276288, 0.55355450,\n",
    "    0.56052689, 0.56204114, 0.56307883, 0.56521540, 0.57830505, 0.57987938,\n",
    "    0.58276149, 0.59040369, 0.59608911, 0.59657872, 0.60755394, 0.61502897,\n",
    "    0.62056515, 0.62125169\n",
    "])\n",
    "\n",
    "y_abs_ref = np.array([\n",
    "    0.0293, 1.1700, 0.0189, 0.0000, 0.0008, 0.0000, 0.0001, 0.0000, 0.0003,\n",
    "    0.0004, 0.0004, 0.0036, 0.0011, 0.0024, 0.0000, 0.0000, 0.0000, 0.0000,\n",
    "    0.0003, 0.0076, 0.0000, 0.0000, 0.0042, 0.0170, 0.0268, 0.0055, 0.2073,\n",
    "    0.7341, 0.7807, 0.8590, 0.0543, 0.0794, 0.0043, 0.0003, 0.0024, 0.1701,\n",
    "    0.0002, 0.0001, 0.0005, 0.0000, 0.0190, 0.4825, 0.0609, 0.0002, 0.0031,\n",
    "    0.0004, 0.2050, 0.0002, 0.0191, 0.0152\n",
    "])\n",
    "\n",
    "y_ecd_ref = np.array([\n",
    "    0.144750, -0.164027, -0.000035, 0.000009, 0.014472, 0.001995, 0.004958,\n",
    "    0.000899, -0.019302, 0.000786, 0.013440, -0.007686, 0.000918, 0.001135,\n",
    "    -0.000005, -0.000019, 0.000222, 0.003878, 0.000392, -0.000790, -0.000002,\n",
    "    0.000000, 0.006479, 0.047084, 0.458896, -0.154583, -0.063050, -0.217650,\n",
    "    -0.459211, 0.383874, 0.041366, -0.032507, 0.000123, 0.000373, 0.025144,\n",
    "    -0.050821, 0.000443, 0.003195, -0.001069, 0.000036, 0.285024, -0.269169,\n",
    "    -0.004591, 0.000050, 0.005488, 0.001347, -0.000665, 0.000083, -0.023592,\n",
    "    -0.011323\n",
    "])\n",
    "\n",
    "x0_ref,y0_ref = lorentzian(x_ref, y_abs_ref, min(x)-1.0, max(x)+1.0, 0.01, 0.2)\n",
    "plt.plot(x0,y0,label='Exciton model')\n",
    "plt.plot(x0_ref,y0_ref,label='TDDFT-TDA')\n",
    "plt.legend()\n",
    "plt.show()\n",
    "\n",
    "x1_ref,y1_ref = lorentzian(x_ref, y_ecd_ref, min(x)-1.0, max(x)+1.0, 0.01, 0.2)\n",
    "plt.plot(x1,y1,label='Exciton model')\n",
    "plt.plot(x1_ref,y1_ref,label='TDDFT-TDA')\n",
    "plt.legend()\n",
    "plt.show()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": []
  }
 ],
 "metadata": {
  "celltoolbar": "Tags",
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.10.12"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 4
}
